# -*- coding: utf-8 -*-
"""Abstract risk model
"""
from __future__ import annotations

from abc import ABC
from abc import abstractmethod
from dataclasses import dataclass
from dataclasses import field
from typing import Dict

import cvxpy as cp


@dataclass
class Model(ABC):
    """Abstract risk model"""

    parameter: Dict[str, cp.Parameter] = field(default_factory=dict)

    @abstractmethod
    def estimate(self, weights, **kwargs):
        """
        Estimate the variance given the portfolio weights
        """

    @abstractmethod
    def update(self, **kwargs):
        """
        Update the data in the risk model
        """

    @abstractmethod
    def constraints(self, weights, **kwargs):
        """
        Return the constraints for the risk model
        """



# -*- coding: utf-8 -*-
from __future__ import annotations

import cvxpy as cp


def minrisk_problem(riskmodel, weights, **kwargs):
    problem = cp.Problem(
        cp.Minimize(riskmodel.estimate(weights, **kwargs)),
        [cp.sum(weights) == 1.0, weights >= 0]
        + riskmodel.constraints(weights, **kwargs),
    )

    return problem
